a.button{
    color: #333;
}

#content #changelist-filter .das-search{
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    boz-sizing: border-box;
    width: 100%;
}

#content #changelist-filter .das-ajaxlistfilter-remove .inline-deletelink .inline-deletelink-icon{
    display: inline-block;
    background: transparent url(../../admin/img/inline-delete.png) no-repeat;
    width: 15px;
    height: 15px;
    border: 0px none;
    outline: 0; /* Remove dotted border around link */
    vertical-align: text-bottom;
}

#content #changelist-filter .das-ajaxlistfilter-remove .inline-deletelink:hover .inline-deletelink-icon{
    background-position: -15px 0;
    cursor: pointer;
}

#content #changelist-filter .das-ajaxlistfilter-search input[type=checkbox]{
    margin-right: 5px;
}

#content #changelist-filter .das-ajaxlistfilter-search .status{
    color: #888;
}

#content #changelist-filter ul.das-ajaxlistfilter-remove{
    margin-bottom: 15px;
}

/* Make the right-hand filter list panel a little easier to see. */
#content #changelist-filter {
    -webkit-box-shadow: 5px 5px 10px 0px rgba(0,0,0,0.5);
    -moz-box-shadow: 5px 5px 10px 0px rgba(0,0,0,0.5);
    box-shadow: 5px 5px 10px 0px rgba(0,0,0,0.5);
}
